//--------------------------------------------------------------
// File     : main.c
// Datum    : 03.04.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : CMSIS_BOOT, M4_CMSIS_CORE
// Funktion : DDS-Funktionsgenerator
// Hinweis  : Diese zwei Files muessen auf 8MHz stehen
//              "cmsis_boot/stm32f4xx.h"
//              "cmsis_boot/system_stm32f4xx.c"
// In Configuration diese Define hinzufgen :
// "STM32F429_439xx" , "__ASSEMBLY__" , "USE_STDPERIPH_DRIVER"
//--------------------------------------------------------------

#include "main.h"
#include "dds.h"

int main(void)
{
  uint32_t gui_redraw=0;

  SystemInit(); // Quarz Einstellungen aktivieren

  // init vom Funktionsgenerator
  dds_init();

  #if DDD_USE_LCD==1
    GUI_CHANGE_t gui_status;
    // init der GUI
    gui_init();
  #endif

  #if DDD_USE_COM==1
    COM_CHANGE_t com_status;
    // init der COM-Schnittstelle
    com_init();
  #endif


  // Frq einstellen
  dds_set_frq(100.0);

  // amplitude einstellen
  dds_set_amplitude(2000);

  // Frq einschalten
  dds_start();

  while(1)
  {
    //----------------------------------
    // LCD und Touch-Funktionen
    // check ob Touch betaetigt
    // und event. DDS neu einstellen
    //----------------------------------
    #if DDD_USE_LCD==1
      gui_status=gui_do(gui_redraw);
      if(gui_status==GUI_CHANGE_FRQ) {
        UB_Led_Toggle(LED_GREEN);
        dds_set_frq(DDS.fsoll);
      }
      else if(gui_status==GUI_CHANGE_AMP) {
        UB_Led_Toggle(LED_GREEN);
        dds_set_amplitude(DDS.amplitude);
      }
      else if(gui_status==GUI_CHANGE_OFF) {
        UB_Led_Toggle(LED_GREEN);
        dds_set_offset(DDS.offset);
      }
      else if(gui_status==GUI_CHANGE_WAVE) {
        UB_Led_Toggle(LED_GREEN);
        dds_set_wave(DDS.wave);
      }
    #endif


    //----------------------------------
    // COM-Funktionen
    // check ob Kommando empfangen
    // und event. DDS neu einstellen
    //----------------------------------
    #if DDD_USE_COM==1
      com_status=com_do();
      if(com_status==COM_CHANGE_FRQ) {
        UB_Led_Toggle(LED_GREEN);
        dds_set_frq(DDS.fsoll);
        gui_redraw=1;
      }
      else if(com_status==COM_CHANGE_AMP) {
        UB_Led_Toggle(LED_GREEN);
        dds_set_amplitude(DDS.amplitude);
        gui_redraw=1;
      }
      else if(com_status==COM_CHANGE_OFF) {
        UB_Led_Toggle(LED_GREEN);
        dds_set_offset(DDS.offset);
        gui_redraw=1;
      }
      else if(com_status==COM_CHANGE_WAVE) {
        UB_Led_Toggle(LED_GREEN);
        dds_set_wave(DDS.wave);
        gui_redraw=1;
      }
      else {
        gui_redraw=0;
      }
    #endif
  }
}

