//--------------------------------------------------------------
// File     : com.c
// Datum    : 03.04.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : USB_CDC
// Funktion : Kommunikation vom Funktionsgenerator
//
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "com.h"


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
COM_CHANGE_t p_cmd_check(void);
void p_cmd_unknown(void);
void p_cmd_help(void);
void p_cmd_get_frq(void);
void p_cmd_get_amp(void);
void p_cmd_get_off(void);
void p_cmd_get_wave(void);
COM_CHANGE_t p_cmd_set_frq(void);
COM_CHANGE_t p_cmd_set_amp(void);
COM_CHANGE_t p_cmd_set_off(void);
COM_CHANGE_t p_cmd_set_wave(void);
void p_cmd_user_wave_width(void);
void p_cmd_user_wave_start(void);
void p_cmd_user_wave_verify(void);
void p_cmd_user_wave_data(void);
COM_CHANGE_t p_cmd_user_wave_stop(void);





//--------------------------------------------------------------
// COM init
//--------------------------------------------------------------
void com_init(void)
{
  // init vom USB_CDC
  UB_USB_CDC_Init();

  COM.user_mode=COM_USER_WAIT;
  COM.user_ptr=0;
  COM.user_error=0;
}



//--------------------------------------------------------------
// COM bearbeiten
//--------------------------------------------------------------
COM_CHANGE_t com_do(void)
{
  COM_CHANGE_t ret_wert=COM_CHANGE_NONE;
  USB_CDC_STATUS_t usb_status;
  USB_CDC_RXSTATUS_t rx_status;

  // test ob USB-Verbindung besteht
  usb_status=UB_USB_CDC_GetStatus();
  if(usb_status==USB_CDC_CONNECTED) {
    // test ob String empfangen
    rx_status=UB_USB_CDC_ReceiveString(com_rx_buf);
    if(rx_status==RX_READY) {
      // Kommando auswerten
      ret_wert=p_cmd_check();
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// Auswertung vom Empfangenen Kommando
//--------------------------------------------------------------
COM_CHANGE_t p_cmd_check(void)
{
  COM_CHANGE_t ret_wert=COM_CHANGE_NONE;

  // Test welches Kommando empfangen wurde
  if(strcmp(com_rx_buf,"HELP")==0) {
    p_cmd_help();
  }
  else if(strcmp(com_rx_buf,"GET_FRQ")==0) {
    p_cmd_get_frq();
  }
  else if(strcmp(com_rx_buf,"GET_AMP")==0) {
    p_cmd_get_amp();
  }
  else if(strcmp(com_rx_buf,"GET_OFF")==0) {
    p_cmd_get_off();
  }
  else if(strcmp(com_rx_buf,"GET_WAVE")==0) {
    p_cmd_get_wave();
  }
  else if(strncmp(com_rx_buf,"SET_FRQ=",8)==0) {
    ret_wert=p_cmd_set_frq();
  }
  else if(strncmp(com_rx_buf,"SET_AMP=",8)==0) {
    ret_wert=p_cmd_set_amp();
  }
  else if(strncmp(com_rx_buf,"SET_OFF=",8)==0) {
    ret_wert=p_cmd_set_off();
  }
  else if(strncmp(com_rx_buf,"SET_WAVE=",9)==0) {
    ret_wert=p_cmd_set_wave();
  }
  else if(strncmp(com_rx_buf,"USER_WAVE_WIDTH=",16)==0) {
    p_cmd_user_wave_width();
  }
  else if(strncmp(com_rx_buf,"USER_WAVE_START",15)==0) {
    p_cmd_user_wave_start();
  }
  else if(strncmp(com_rx_buf,"DA=",3)==0) {
    p_cmd_user_wave_data();
  }
  else if(strncmp(com_rx_buf,"USER_WAVE_VERIFY",16)==0) {
    p_cmd_user_wave_verify();
  }
  else if(strncmp(com_rx_buf,"VF=",3)==0) {
    p_cmd_user_wave_data();
  }
  else if(strncmp(com_rx_buf,"USER_WAVE_STOP",14)==0) {
    ret_wert=p_cmd_user_wave_stop();
  }
  else {
    p_cmd_unknown();
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// unbekanntes oder Fehlerhaftes Kommando empfangen
//--------------------------------------------------------------
void p_cmd_unknown(void)
{
  UB_USB_CDC_SendString("CMD=?",COM_END_BYTE);
}

//--------------------------------------------------------------
// interne Funktion
// Antwort auf "Help"
//--------------------------------------------------------------
void p_cmd_help(void)
{
  sprintf(com_tx_buf,"DDS=OK [%s]",DDS_VERSION);
  UB_USB_CDC_SendString(com_tx_buf,COM_END_BYTE);
  UB_USB_CDC_SendString("GET_FRQ",COM_END_BYTE);
  UB_USB_CDC_SendString("GET_AMP",COM_END_BYTE);
  UB_USB_CDC_SendString("GET_OFF",COM_END_BYTE);
  UB_USB_CDC_SendString("GET_WAVE",COM_END_BYTE);
  UB_USB_CDC_SendString("SET_FRQ=a.b",COM_END_BYTE);
  UB_USB_CDC_SendString("SET_AMP=a",COM_END_BYTE); 
  UB_USB_CDC_SendString("SET_OFF=a",COM_END_BYTE);
  UB_USB_CDC_SendString("SET_WAVE=a",COM_END_BYTE);
}

//--------------------------------------------------------------
// interne Funktion
// aktuelle FRQ senden
//--------------------------------------------------------------
void p_cmd_get_frq(void)
{
  P_FloatToDezStr(DDS.fsoll);
  sprintf(com_tx_buf,"FRQ=%s",bval);
  UB_USB_CDC_SendString(com_tx_buf,COM_END_BYTE);
}

//--------------------------------------------------------------
// interne Funktion
// aktuelle Amlpitude senden
//--------------------------------------------------------------
void p_cmd_get_amp(void)
{
  sprintf(com_tx_buf,"AMP=%d",DDS.amplitude);
  UB_USB_CDC_SendString(com_tx_buf,COM_END_BYTE);
}

//--------------------------------------------------------------
// interne Funktion
// aktuellen Offset senden
//--------------------------------------------------------------
void p_cmd_get_off(void)
{
  sprintf(com_tx_buf,"OFF=%d",DDS.offset);
  UB_USB_CDC_SendString(com_tx_buf,COM_END_BYTE);
}


//--------------------------------------------------------------
// interne Funktion
// aktuelle Kurvenform senden
//--------------------------------------------------------------
void p_cmd_get_wave(void)
{
  sprintf(com_tx_buf,"WAVE=%d",DDS.wave);
  UB_USB_CDC_SendString(com_tx_buf,COM_END_BYTE);
}

//--------------------------------------------------------------
// interne Funktion
// FRQ einstellen
// "SET_FRQ=12345.456"
//--------------------------------------------------------------
COM_CHANGE_t p_cmd_set_frq(void)
{
  uint8_t len,ok,n,pos;
  char c;
  char zahl[20];
  float f_val;

  len=strlen(com_rx_buf);

  // check ob laenge ok
  if((len<9) || (len>18)) {
    p_cmd_unknown();
    return(COM_CHANGE_NONE);
  }

  // check ob alle Zeichen ok
  ok=0;
  pos=0;
  for(n=8;n<len;n++) {
    c=com_rx_buf[n];
    zahl[pos]=c;
    pos++;
    if((c<48) || (c>57)) {
      if(c!=46) ok=1;
    }
  }
  zahl[pos]=0x00;

  if(ok==1) {
    p_cmd_unknown();
    return(COM_CHANGE_NONE);
  }

  // string in float wandeln
  f_val=atof(zahl);

  // pruefen auf Grenzwerte
  if(f_val<DDS_MIN_FRQ) f_val=DDS_MIN_FRQ;
  if(f_val>DDS_MAX_FRQ) f_val=DDS_MAX_FRQ;

  DDS.fsoll=f_val;

  // OK-Meldung ausgeben
  P_FloatToDezStr(DDS.fsoll);
  sprintf(com_tx_buf,"SET_FRQ_OK=%s",bval);
  UB_USB_CDC_SendString(com_tx_buf,COM_END_BYTE);

  return(COM_CHANGE_FRQ);
}


//--------------------------------------------------------------
// interne Funktion
// Amplitude einstellen
// "SET_AMP=1234"
//--------------------------------------------------------------
COM_CHANGE_t p_cmd_set_amp(void)
{
  uint8_t len,ok,n,pos;
  char c;
  char zahl[20];
  uint16_t u_val;

  len=strlen(com_rx_buf);

  // check ob laenge ok
  if((len<9) || (len>12)) {
    p_cmd_unknown();
    return(COM_CHANGE_NONE);
  }

  // check ob alle Zeichen ok
  ok=0;
  pos=0;
  for(n=8;n<len;n++) {
    c=com_rx_buf[n];
    zahl[pos]=c;
    pos++;
    if((c<48) || (c>57)) {
      ok=1;
    }
  }
  zahl[pos]=0x00;

  if(ok==1) {
    p_cmd_unknown();
    return(COM_CHANGE_NONE);
  }

  // string in integer wandeln
  u_val=atoi(zahl);

  // pruefen auf Grenzwerte
  if(u_val>DDS_MAX_DAC_WERT) u_val=DDS_MAX_DAC_WERT;

  DDS.amplitude=u_val;

  // OK-Meldung ausgeben
  sprintf(com_tx_buf,"SET_AMP_OK=%d",DDS.amplitude);
  UB_USB_CDC_SendString(com_tx_buf,COM_END_BYTE);

  return(COM_CHANGE_AMP);
}



//--------------------------------------------------------------
// interne Funktion
// Offset einstellen
// "SET_OFF=1234"
//--------------------------------------------------------------
COM_CHANGE_t p_cmd_set_off(void)
{
  uint8_t len,ok,n,pos;
  char c;
  char zahl[20];
  uint16_t u_val;

  len=strlen(com_rx_buf);

  // check ob laenge ok
  if((len<9) || (len>12)) {
    p_cmd_unknown();
    return(COM_CHANGE_NONE);
  }

  // check ob alle Zeichen ok
  ok=0;
  pos=0;
  for(n=8;n<len;n++) {
    c=com_rx_buf[n];
    zahl[pos]=c;
    pos++;
    if((c<48) || (c>57)) {
      ok=1;
    }
  }
  zahl[pos]=0x00;

  if(ok==1) {
    p_cmd_unknown();
    return(COM_CHANGE_NONE);
  }

  // string in integer wandeln
  u_val=atoi(zahl);

  // pruefen auf Grenzwerte
  if(u_val>DDS_MAX_DAC_WERT) u_val=DDS_MAX_DAC_WERT;

  DDS.offset=u_val;

  // OK-Meldung ausgeben
  sprintf(com_tx_buf,"SET_OFF_OK=%d",DDS.offset);
  UB_USB_CDC_SendString(com_tx_buf,COM_END_BYTE);

  return(COM_CHANGE_OFF);
}


//--------------------------------------------------------------
// interne Funktion
// Kurvenform einstellen
// "SET_WAVE=1"
//--------------------------------------------------------------
COM_CHANGE_t p_cmd_set_wave(void)
{
  uint8_t len;
  char c;
  char zahl[20];
  uint16_t u_val;

  len=strlen(com_rx_buf);

  // check ob laenge ok
  if(len!=10) {
    p_cmd_unknown();
    return(COM_CHANGE_NONE);
  }

  // check ob alle Zeichen ok
  c=com_rx_buf[9];
  zahl[0]=c;
  if((c<48) || (c>57)) {
    p_cmd_unknown();
    return(COM_CHANGE_NONE);
  }
  zahl[1]=0x00;

  // string in integer wandeln
  u_val=atoi(zahl);

  // pruefen auf Grenzwerte
  if(u_val>DDS_MAX_WAVEFORM) u_val=DDS_MAX_WAVEFORM;

  DDS.wave=u_val;

  // OK-Meldung ausgeben
  sprintf(com_tx_buf,"SET_WAVE_OK=%d",DDS.wave);
  UB_USB_CDC_SendString(com_tx_buf,COM_END_BYTE);

  return(COM_CHANGE_WAVE);
}


//--------------------------------------------------------------
// interne Funktion
// User-Waveform (bit breite)
// "USER_WAVE_WIDTH=7"
//--------------------------------------------------------------
void p_cmd_user_wave_width(void)
{
  uint8_t len,ok,n,pos;
  char c;
  char zahl[20];
  uint16_t u_val;

  if(COM.user_mode!=COM_USER_WAIT) {
    COM.user_error++;
    p_cmd_unknown();
    return;
  }

  len=strlen(com_rx_buf);

  // check ob laenge ok
  if((len<17) || (len>18)) {
    COM.user_error++;
    p_cmd_unknown();
    return;
  }

  // check ob alle Zeichen ok
  ok=0;
  pos=0;
  for(n=16;n<len;n++) {
    c=com_rx_buf[n];
    zahl[pos]=c;
    pos++;
    if((c<48) || (c>57)) {
      ok=1;
    }
  }
  zahl[pos]=0x00;

  if(ok==1) {
    COM.user_error++;
    p_cmd_unknown();
    return;
  }

  // string in integer wandeln
  u_val=atoi(zahl);

  // pruefen auf Grenzwerte
  if(u_val<1) u_val=1;
  if(u_val>DDS_MAX_USER_WIDTH) u_val=DDS_MAX_USER_WIDTH;

  DDS.user_width=u_val;
  if(DDS.user_width==1) DDS.user_len=2;
  if(DDS.user_width==2) DDS.user_len=4;
  if(DDS.user_width==3) DDS.user_len=8;
  if(DDS.user_width==4) DDS.user_len=16;
  if(DDS.user_width==5) DDS.user_len=32;
  if(DDS.user_width==6) DDS.user_len=64;
  if(DDS.user_width==7) DDS.user_len=128;
  if(DDS.user_width==8) DDS.user_len=256;
  if(DDS.user_width==9) DDS.user_len=512;
  if(DDS.user_width==10) DDS.user_len=1024;
  if(DDS.user_width==11) DDS.user_len=2048;
  if(DDS.user_width==12) DDS.user_len=4096;

  // OK-Meldung ausgeben
  sprintf(com_tx_buf,"USER_WAVE_WIDTH_OK=%d",DDS.user_width);
  UB_USB_CDC_SendString(com_tx_buf,COM_END_BYTE);
}


//--------------------------------------------------------------
// interne Funktion
// User-Waveform (start)
//--------------------------------------------------------------
void p_cmd_user_wave_start(void)
{
  if(COM.user_mode==COM_USER_WAIT) {
    COM.user_mode=COM_USER_DATA; 
    COM.user_error=0;
    dds_clear_user_wave();   
    UB_USB_CDC_SendString("UW_START_OK",COM_END_BYTE);
  }
  else {
    p_cmd_unknown();
  }
}


//--------------------------------------------------------------
// interne Funktion
// User-Waveform (verify)
//--------------------------------------------------------------
void p_cmd_user_wave_verify(void)
{
  if(COM.user_mode==COM_USER_DATA) {
    COM.user_mode=COM_USER_VERIFY; 
    COM.user_ptr=0;               
    UB_USB_CDC_SendString("UW_VERIFY_OK",COM_END_BYTE);
  }
  else {
    p_cmd_unknown();
  }
}

//--------------------------------------------------------------
// interne Funktion
// User-Waveform Daten (speichern oder verify)
// "DA=1234" oder "VF=1234"
//--------------------------------------------------------------
void p_cmd_user_wave_data(void)
{
  uint8_t len,ok,n,pos;
  char c;
  char zahl[20];
  uint16_t u_val;


  if(COM.user_mode==COM_USER_WAIT) {
    COM.user_error++;
    p_cmd_unknown();
    return;
  }

  len=strlen(com_rx_buf);

  // check ob laenge ok
  if((len<4) || (len>7)) {
    COM.user_error++;
    p_cmd_unknown();
    return;
  }

  // check ob alle Zeichen ok
  ok=0;
  pos=0;
  for(n=3;n<len;n++) {
    c=com_rx_buf[n];
    zahl[pos]=c;
    pos++;
    if((c<48) || (c>57)) {
      ok=1;
    }
  }
  zahl[pos]=0x00;

  if(ok==1) {
    COM.user_error++;
    p_cmd_unknown();
    return;
  }

  // string in integer wandeln
  u_val=atoi(zahl);

  // pruefen auf Grenzwerte
  if(u_val>DDS_MAX_DAC_WERT) u_val=DDS_MAX_DAC_WERT;

  if(COM.user_mode==COM_USER_DATA) {
    // Wert speichern
    dds_add_user_wave(u_val);
  }
  else if(COM.user_mode==COM_USER_VERIFY) {
    // verify
    if(DDS_USER[COM.user_ptr]!=u_val) {
      // fehler beim verify
      COM.user_error++;
    }
    COM.user_ptr++;
    if(COM.user_ptr>=DDS_MAX_USER_LEN) COM.user_ptr=0;
  }
}


//--------------------------------------------------------------
// interne Funktion
// User-Waveform (stop)
//--------------------------------------------------------------
COM_CHANGE_t p_cmd_user_wave_stop(void)
{
  if(COM.user_mode==COM_USER_DATA) {
    COM.user_mode=COM_USER_WAIT;
    UB_USB_CDC_SendString("UW_STOP_OK",COM_END_BYTE);
    if(COM.user_error==0) {
      UB_USB_CDC_SendString("DATA=OK",COM_END_BYTE);
    }
    else {
      UB_USB_CDC_SendString("DATA=ERR",COM_END_BYTE);
    }
  }
  else if(COM.user_mode==COM_USER_VERIFY) {
    COM.user_mode=COM_USER_WAIT;
    UB_USB_CDC_SendString("UW_STOP_OK",COM_END_BYTE);
    if(COM.user_error==0) {
      UB_USB_CDC_SendString("VERIFY=OK",COM_END_BYTE);
    }
    else {
      UB_USB_CDC_SendString("VERIFY=ERR",COM_END_BYTE);
    }
  }
  else {
    p_cmd_unknown();
    return(COM_CHANGE_NONE);
  }

  if(COM.user_error!=0) {
    // bei fehler alles loeschen
    dds_clear_user_wave();
  }

  if(DDS.wave!=4) {
    return(COM_CHANGE_NONE);
  }

  // damit User-Kurvenform aktuallisiert wird
  return(COM_CHANGE_WAVE);
}
