//--------------------------------------------------------------
// File     : dds.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_DDS_H
#define __STM32F4_UB_DDS_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32_ub_led.h"
#include "stm32_ub_tim2.h"
#include "stm32_ub_dac.h"


//--------------------------------------------------------------
// Versions-Nummer (als String)
//--------------------------------------------------------------
#define   DDS_VERSION  "V:1.00"



//--------------------------------------------------------------
// Benutzung vom LCD/Touch kann hier abgeschaltet werden
// (ohne LCD -> Steuerung nur per Com-Schnittstelle)
//--------------------------------------------------------------
//#define   DDS_USE_LCD   0  // ohne LCD+Touch
#define   DDD_USE_LCD   1  // mit LCD+Touch


#if DDD_USE_LCD==1
  #include "gui.h"
#endif



//--------------------------------------------------------------
// Benutzung der COM-Schnittstelle kann hier abgeschaltet werden
// (ohne COM -> Steuerung nur per LCD/Touch)
//--------------------------------------------------------------
//#define   DDS_USE_COM   0  // ohne COM
#define   DDD_USE_COM   1  // mit COM


#if DDD_USE_COM==1
  #include "com.h"
#endif


//--------------------------------------------------------------
#if DDD_USE_LCD == 0 && DDD_USE_COM==0
#error aktivate LCD or COM.
#endif



//--------------------------------------------------------------
// Array fuer Float->String
//--------------------------------------------------------------
char bval[15];

//--------------------------------------------------------------
// Anzahl der Nachkommastellen bei Float
//  Faktor ->  100 = 2 Nachkommastellen,  Formatierung -> "%d.%02d"
//  Faktor -> 1000 = 3 Nachkommastellen,  Formatierung -> "%d.%03d"
//  usw
//--------------------------------------------------------------
#define  STRING_FLOAT_FAKTOR     1000    // 1000 = 3 Nachkommastellen
#define  STRING_FLOAT_FORMAT "%d.%03d"  // Formatierung



//--------------------------------------------------------------
// FRQ-Grenzen der DDS
//--------------------------------------------------------------
#define   DDS_STANDARD_FRQ   100.0   // 100 Hz
#define   DDS_MIN_FRQ          1.0   // minimum (in Hz)
#define   DDS_MAX_FRQ      50000.0   // maximum (in Hz)




//--------------------------------------------------------------
// DAC-Settings der DDS
//--------------------------------------------------------------
#define   DDS_STANDARD_DAC_WERT  2000
#define   DDS_MAX_DAC_WERT       4095  // maximum DAC-Wert = 12bit



//--------------------------------------------------------------
// Kurvenformen (im Flash)
//--------------------------------------------------------------
#define   DDS_STANDARD_WAVE    0
#define   DDS_MAX_WAVEFORM     4 // anzahl der Kurvenformen

// Nr. 0 = Sinus (im Flash)
#define   DDS_WAVE_SIN_WIDTH       8 // Breite = 8bit
#define   DDS_WAVE_SIN_LEN       256 // 2^8 = 256
// Nr. 1 = Rechteck (im Flash)
#define   DDS_WAVE_SQ_WIDTH        3 // Breite = 3bit
#define   DDS_WAVE_SQ_LEN          8 // 2^3 = 8
// Nr. 2 = Sgezahn (im Flash)
#define   DDS_WAVE_SAW_WIDTH       7 // Breite = 7bit
#define   DDS_WAVE_SAW_LEN       128 // 2^7 = 128
// Nr. 3 = Dreieck (im Flash)
#define   DDS_WAVE_TRI_WIDTH       8 // Breite = 8bit
#define   DDS_WAVE_TRI_LEN       256 // 2^8 = 256
// Nr. 4 = User-Kurvenform (im RAM)





//--------------------------------------------------------------
// User Kurvenform
//--------------------------------------------------------------
#define   DDS_MAX_USER_WIDTH   12  // breite max 12bit
#define   DDS_MAX_USER_LEN   4096  // max Laenge (2^bit)
uint16_t DDS_USER[DDS_MAX_USER_LEN];




//--------------------------------------------------------------
// Timer Einstellungen
// TIM2_FRQ = 84MHz
// DDS_FRQ (Abtastfrq) = 84MHz / (Prescale+1) / (Periode+1)
//
// Eingestellt : DDS_FRQ = 250kHz (4us)
//--------------------------------------------------------------
#define  DDS_TIM2_PRESCALE  20  // 84MHz / 21 = 4MHz
#define  DDS_TIM2_PERIOD    15  //  4MHz / 16 = 250kHz

#define  DDS_FRQ   250000.0   // 250kHz (als float)




//--------------------------------------------------------------
// Phasencounter
// n-bit
// Maxwert = 2^n
//--------------------------------------------------------------
#define  DDS_PHASE_WIDTH           32  // Breite = 32bit
#define  DDS_PHASE_MAX   4294967296.0  // Max = 2^Bit (als float)
 



//--------------------------------------------------------------
// Daten-Array (das ausgegeben wird)
// n-bit
// Laenge = 2^n
//--------------------------------------------------------------
#define  DDS_DATA_ARRAY_WIDTH     12  // breite 12bit
#define  DDS_DATA_ARRAY_LEN     4096  // Len = 2^Bit

uint16_t DDS_DATA_ARRAY[DDS_DATA_ARRAY_LEN];




//--------------------------------------------------------------
// Struktur vom DDS-Funktionsgenerator
//--------------------------------------------------------------
typedef struct {
  uint32_t phase_accu;   // 32bit Phasenaccumulator (Zhler)
  uint32_t delta;        // Schrittweite vom Zhler
  uint32_t bitshift;     // shift anzahl
  double fsoll;          // Frequenz Sollwert
  uint16_t amplitude;    // Amplituden Sollwert (0...4095)
  uint16_t offset;       // Offset Sollwert (0...4095)
  uint16_t wave;         // Aktuelle Kurvenform (0...4)
  uint16_t user_ptr;     // User-Kurvenform Pointer
  uint16_t user_width;   // User-Kurvenform (bit breite)
  uint32_t user_len;     // User-Kurvenform laenge
}DDS_t;
DDS_t DDS;



//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
void dds_init(void);
void dds_start(void);
void dds_stop(void);
void dds_set_frq(double frq);
void dds_set_amplitude(uint16_t amplitude);
void dds_set_offset(uint16_t offset);
void dds_set_wave(uint16_t wave);
void dds_clear_user_wave(void);
void dds_add_user_wave(uint16_t wert);
void P_FloatToDezStr(float wert);



//--------------------------------------------------------------
#endif // __STM32F4_UB_DDS_H
