//--------------------------------------------------------------
// File     : gui.c
// Datum    : 03.04.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : ILI9341, FONT, GRAPHIC2D, TOUCH, SYSTICK
//            retarget_printf
// Funktion : GUI vom Funktionsgenerator
//
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "gui.h"


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void p_menu_check_TOUCH(void);
void p_menu_draw_GUI(void);
void p_menu_draw_FRQ(void);
void p_menu_draw_WAVEFORM(void);
void p_menu_draw_AMP(void);
void p_menu_draw_OFF(void);
GUI_CHANGE_t p_change_value(uint32_t mode);


//--------------------------------------------------------------
// Globale Buffer (fr printf)
//--------------------------------------------------------------
char buf[30];





//--------------------------------------------------------------
// init der GUI
//--------------------------------------------------------------
void gui_init(void)
{
  // init aller Variablen
  GUI.redraw=GUI_REDRAW_ALL;
  GUI.akt_mode=GUI_MODE_FRQ;
  GUI.akt_button=GUI_BTN_NONE;
  GUI.old_button=GUI_BTN_NONE;

  // init vom Systick
  UB_Systick_Init();

  // init vom Display
  UB_LCD_Init();
  // auf Landscape umschalten
  UB_LCD_SetMode(LANDSCAPE);

  // Display einschalten
  UB_LCD_LayerInit_Fullscreen();
  UB_LCD_SetLayer_2();
  UB_LCD_FillLayer(RGB_COL_WHITE);

  // init vom Touch
  if(UB_Touch_Init()!=SUCCESS) {
    // Status Text ausgeben
    UB_Font_DrawString(0,0,"Touch error, please restart...",&Arial_10x15,RGB_COL_WHITE,RGB_COL_RED);
    while(1);
  }

  // Status Texte ausgeben
  UB_Font_DrawString(GUI_HEADER_XP,GUI_HEADER_YP,"DDS-Waveform-Generator",&Arial_10x15,RGB_COL_WHITE,RGB_COL_RED);
  UB_Font_DrawString(GUI_VERSION_XP,GUI_VERSION_YP,DDS_VERSION,&Arial_10x15,RGB_COL_BLACK,RGB_COL_WHITE);
  UB_Font_DrawString(GUI_FOOTER_XP,GUI_FOOTER_YP,"Out=PA5  COM=USB_OTG",&Arial_10x15,RGB_COL_BLACK,RGB_COL_WHITE);
  UB_Font_DrawString(GUI_FORM_HEADER_XPOS,GUI_FORM_HEADER_YPOS,"Waveform",&Arial_14x22,RGB_COL_BLACK,RGB_COL_WHITE);
  UB_Font_DrawString(GUI_FRQ_HEADER_XPOS,GUI_FRQ_HEADER_YPOS,"Freqency [Hz]",&Arial_14x22,RGB_COL_BLACK,RGB_COL_WHITE);
  UB_Font_DrawString(GUI_AMP_HEADER_XPOS,GUI_AMP_HEADER_YPOS,"Ampl.",&Arial_14x22,RGB_COL_BLACK,RGB_COL_WHITE);
  UB_Font_DrawString(GUI_OFF_HEADER_XPOS,GUI_OFF_HEADER_YPOS,"Offset",&Arial_14x22,RGB_COL_BLACK,RGB_COL_WHITE);

  GUI_Counter=GUI_DELAY_MS;
}


//--------------------------------------------------------------
// GUI bearbeiten
// redraw : 0=GUI nicht neu zeichnen
//          1=GUI komplett neu zeichnen
//--------------------------------------------------------------
GUI_CHANGE_t gui_do(uint32_t redraw)
{
  GUI_CHANGE_t ret_wert=GUI_CHANGE_NONE;
  static uint32_t updown=0;


  // check ob refreshzeit abgelaufen
  if((GUI_Counter==0) || (redraw!=0)) {
    GUI_Counter=GUI_DELAY_MS;

    if(redraw!=0) GUI.redraw|=GUI_REDRAW_ALL;

    // check Touch
    p_menu_check_TOUCH();

    if(GUI.akt_button==GUI_BTN_RIGHT) {
      // button-right betaetigt
      ret_wert=p_change_value(1);
    }
    else if(GUI.akt_button==GUI_BTN_LEFT) {
      // button-left betaetigt
      ret_wert=p_change_value(2);
    }
    else if(GUI.akt_button==GUI_BTN_UP) {
      // button-up betaetigt
      if(updown==0) {
        if(GUI.akt_mode==GUI_MODE_FRQ) GUI.akt_mode=GUI_MODE_WAVE;
        if(GUI.akt_mode==GUI_MODE_AMPLITUDE) GUI.akt_mode=GUI_MODE_FRQ;
        if(GUI.akt_mode==GUI_MODE_OFFSET) GUI.akt_mode=GUI_MODE_AMPLITUDE;
        GUI.redraw|=GUI_REDRAW_ALL;
        updown=1;
      }
    }
    else if(GUI.akt_button==GUI_BTN_DOWN) {
      // button-down betaetigt
      if(updown==0) {
        if(GUI.akt_mode==GUI_MODE_AMPLITUDE) GUI.akt_mode=GUI_MODE_OFFSET;
        if(GUI.akt_mode==GUI_MODE_FRQ) GUI.akt_mode=GUI_MODE_AMPLITUDE;
        if(GUI.akt_mode==GUI_MODE_WAVE) GUI.akt_mode=GUI_MODE_FRQ;
        GUI.redraw|=GUI_REDRAW_ALL;
        updown=1;
      }
    }
    else {
      // kein Button betaetigt
      p_change_value(0);
      updown=0;
    }

    // event GUI neu zeichnen
    if(GUI.redraw!=0) {
      if((GUI.redraw&GUI_REDRAW_BTN)!=0) p_menu_draw_GUI();
      if((GUI.redraw&GUI_REDRAW_FRQ)!=0) p_menu_draw_FRQ();
      if((GUI.redraw&GUI_REDRAW_WAVE)!=0) p_menu_draw_WAVEFORM();
      if((GUI.redraw&GUI_REDRAW_AMP)!=0) p_menu_draw_AMP();
      if((GUI.redraw&GUI_REDRAW_OFF)!=0) p_menu_draw_OFF();

      GUI.redraw=0;
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// aktiven Value aendern
// mode : 0=kein button betaetigt
//        1=incrementieren
//        2=decrementieren
//--------------------------------------------------------------
GUI_CHANGE_t p_change_value(uint32_t mode)
{
  GUI_CHANGE_t ret_wert=GUI_CHANGE_NONE;
  static uint32_t next_step=0;
  static float frq_step=0.001;
  static uint16_t amp_step=1;
  int16_t amp_wert; // muss signed int sein !!

  if(mode==0) {
    next_step=0;
    frq_step=0.001;
    amp_step=1;
  }
  else if(mode==1) {
    if(GUI.akt_mode==GUI_MODE_FRQ) {
      DDS.fsoll+=frq_step;
      if(DDS.fsoll>DDS_MAX_FRQ) DDS.fsoll=DDS_MAX_FRQ;

      ret_wert=GUI_CHANGE_FRQ;
      GUI.redraw|=GUI_REDRAW_FRQ;
      next_step++;
      if(next_step>50) {
        next_step=0;
        frq_step*=10;
      }
    }
    else if(GUI.akt_mode==GUI_MODE_AMPLITUDE) {
      amp_wert=DDS.amplitude+amp_step;
      if(amp_wert>DDS_MAX_DAC_WERT) amp_wert=DDS_MAX_DAC_WERT;
      DDS.amplitude=amp_wert;

      ret_wert=GUI_CHANGE_AMP;
      GUI.redraw|=GUI_REDRAW_AMP;
      next_step++;
      if(next_step>50) {
        next_step=0;
        amp_step*=10;
      }
    }
    else if(GUI.akt_mode==GUI_MODE_OFFSET) {
      amp_wert=DDS.offset+amp_step;
      if(amp_wert>DDS_MAX_DAC_WERT) amp_wert=DDS_MAX_DAC_WERT;
      DDS.offset=amp_wert;

      ret_wert=GUI_CHANGE_OFF;
      GUI.redraw|=GUI_REDRAW_OFF;
      next_step++;
      if(next_step>50) {
        next_step=0;
        amp_step*=10;
      }
    }
    else if(GUI.akt_mode==GUI_MODE_WAVE) {
      if(DDS.wave<DDS_MAX_WAVEFORM) {
        if(next_step==0) {
          next_step=1;
          DDS.wave++;
          ret_wert=GUI_CHANGE_WAVE;
          GUI.redraw|=GUI_REDRAW_WAVE;
        }
      }
    }
  }
  else if(mode==2) {
    if(GUI.akt_mode==GUI_MODE_FRQ) {
      DDS.fsoll-=frq_step;
      if(DDS.fsoll<DDS_MIN_FRQ) DDS.fsoll=DDS_MIN_FRQ;
      
      ret_wert=GUI_CHANGE_FRQ;
      GUI.redraw|=GUI_REDRAW_FRQ;
      next_step++;
      if(next_step>50) {
        next_step=0;
        frq_step*=10;
      }
    }
    else if(GUI.akt_mode==GUI_MODE_AMPLITUDE) {
      amp_wert=DDS.amplitude-amp_step;
      if(amp_wert<0) amp_wert=0;
      DDS.amplitude=amp_wert;

      ret_wert=GUI_CHANGE_AMP;
      GUI.redraw|=GUI_REDRAW_AMP;
      next_step++;
      if(next_step>50) {
        next_step=0;
        amp_step*=10;
      }
    }
    else if(GUI.akt_mode==GUI_MODE_OFFSET) {
      amp_wert=DDS.offset-amp_step;
      if(amp_wert<0) amp_wert=0;
      DDS.offset=amp_wert;

      ret_wert=GUI_CHANGE_OFF;
      GUI.redraw|=GUI_REDRAW_OFF;
      next_step++;
      if(next_step>50) {
        next_step=0;
        amp_step*=10;
      }
    }
    else if(GUI.akt_mode==GUI_MODE_WAVE) {
      if(DDS.wave>0) {
        if(next_step==0) {
          next_step=1;
          DDS.wave--;
          ret_wert=GUI_CHANGE_WAVE;
          GUI.redraw|=GUI_REDRAW_WAVE;
        }
      }
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// check ob Touch betaetigt ist
//--------------------------------------------------------------
void p_menu_check_TOUCH(void)
{
  uint16_t x,y;
  static uint16_t x_old=999,y_old=999;

  UB_Touch_Read();
  if(Touch_Data.status==TOUCH_PRESSED) {
    // Touch ist bettigt
    x=Touch_Data.xp;
    y=Touch_Data.yp;

    if((x!=x_old) || (y!=y_old)) {
      x_old=x;
      y_old=y;
    }
    else {
      if(y>GUI_XPOS) {
        if(x>(GUI_YPOS+(GUI1.width/2))) {
          // left/right
          if(y>(GUI_XPOS+(GUI1.height/2))) {
            GUI.akt_button=GUI_BTN_RIGHT;
          }
          else {
            GUI.akt_button=GUI_BTN_LEFT;
          }
        }
        else {
          // up/down
          if(x>(GUI_YPOS+(GUI1.width/4))) {
            GUI.akt_button=GUI_BTN_UP;
          }
          else {
            GUI.akt_button=GUI_BTN_DOWN;
          }
        }
      }
      else {
        GUI.akt_button=GUI_BTN_NONE;
      }
    }
  }
  else {
    // Touch ist nicht bettigt
    GUI.akt_button=GUI_BTN_NONE;
  }

  if(GUI.akt_button!=GUI.old_button) {
    GUI.redraw|=GUI_REDRAW_BTN;
    GUI.old_button=GUI.akt_button;
  }
}

//--------------------------------------------------------------
// interne Funkion
// zeichnet die GUI
// (zeichnet auch den betaetigten Button)
//--------------------------------------------------------------
void p_menu_draw_GUI(void)
{
  DMA2D_Koord koord;

  //--------------------------
  // leere GUI zeichnen
  //--------------------------
  koord.source_xp=0;
  koord.source_yp=0;
  koord.source_w=GUI1.width;
  koord.source_h=GUI1.height;
  koord.dest_xp=GUI_YPOS;
  koord.dest_yp=GUI_XPOS;

  UB_Graphic2D_CopyImgDMA(&GUI1,koord);


  //--------------------------
  // betaetigten Button zeichnen
  //--------------------------
  if(GUI.akt_button==GUI_BTN_RIGHT) {
    koord.source_xp=GUI1.width/2;
    koord.source_yp=GUI1.height/2;
    koord.source_w=GUI1.width/2;
    koord.source_h=GUI1.height/2;
    koord.dest_xp=GUI_YPOS+(GUI1.width/2);
    koord.dest_yp=GUI_XPOS+(GUI1.height/2);
    UB_Graphic2D_CopyImgDMA(&GUI2,koord);
  }
  else if(GUI.akt_button==GUI_BTN_LEFT) {
    koord.source_xp=GUI1.width/2;
    koord.source_yp=0;
    koord.source_w=GUI1.width/2;
    koord.source_h=GUI1.height/2;
    koord.dest_xp=GUI_YPOS+(GUI1.width/2);
    koord.dest_yp=GUI_XPOS;
    UB_Graphic2D_CopyImgDMA(&GUI2,koord);
  }
  else if(GUI.akt_button==GUI_BTN_UP) {
    koord.source_xp=GUI1.width/4;
    koord.source_yp=0;
    koord.source_w=GUI1.width/4;
    koord.source_h=GUI1.height;
    koord.dest_xp=GUI_YPOS+(GUI1.width/4);
    koord.dest_yp=GUI_XPOS;
    UB_Graphic2D_CopyImgDMA(&GUI2,koord);
  }
  else if(GUI.akt_button==GUI_BTN_DOWN) {
    koord.source_xp=0;
    koord.source_yp=0;
    koord.source_w=GUI1.width/4;
    koord.source_h=GUI1.height;
    koord.dest_xp=GUI_YPOS;
    koord.dest_yp=GUI_XPOS;
    UB_Graphic2D_CopyImgDMA(&GUI2,koord);
  }
}

//--------------------------------------------------------------
// interne Funktion
// anzeige der FRQ
//--------------------------------------------------------------
void p_menu_draw_FRQ(void)
{
  static uint32_t old_len=99;

  P_FloatToDezStr(DDS.fsoll);
  sprintf(buf,"%s",bval);
  if(old_len!=strlen(buf)) {
    old_len=strlen(buf);
    UB_Font_DrawString(GUI_FRQ_XPOS,GUI_FRQ_YPOS,"         ",&Arial_14x22,RGB_COL_BLACK,GUI_COL_NORMAL);
  }

  if(GUI.akt_mode==GUI_MODE_FRQ) {
    UB_Font_DrawString(GUI_FRQ_XPOS,GUI_FRQ_YPOS,buf,&Arial_14x22,RGB_COL_BLACK,GUI_COL_AKTIV);
  }
  else {
    UB_Font_DrawString(GUI_FRQ_XPOS,GUI_FRQ_YPOS,buf,&Arial_14x22,RGB_COL_BLACK,GUI_COL_NORMAL);
  }
}


//--------------------------------------------------------------
// interne Funktion
// anzeige der Wellenform
//--------------------------------------------------------------
void p_menu_draw_WAVEFORM(void)
{
  if(DDS.wave==0) strcpy(buf,"0=sine    ");
  if(DDS.wave==1) strcpy(buf,"1=square  ");
  if(DDS.wave==2) strcpy(buf,"2=sawtooth");
  if(DDS.wave==3) strcpy(buf,"3=triangle");
  if(DDS.wave==4) strcpy(buf,"4=user    ");

  if(GUI.akt_mode==GUI_MODE_WAVE) {
    UB_Font_DrawString(GUI_FORM_XPOS,GUI_FORM_YPOS,buf,&Arial_14x22,RGB_COL_BLACK,GUI_COL_AKTIV);
  }
  else {
    UB_Font_DrawString(GUI_FORM_XPOS,GUI_FORM_YPOS,buf,&Arial_14x22,RGB_COL_BLACK,GUI_COL_NORMAL);
  }
}


//--------------------------------------------------------------
// interne Funktion
// anzeige der Amplitude
//--------------------------------------------------------------
void p_menu_draw_AMP(void)
{
  static uint32_t old_len=99;

  sprintf(buf,"%d",DDS.amplitude);
  if(old_len!=strlen(buf)) {
    old_len=strlen(buf);
    UB_Font_DrawString(GUI_AMP_XPOS,GUI_AMP_YPOS,"    ",&Arial_14x22,RGB_COL_BLACK,GUI_COL_NORMAL);
  }

  if(GUI.akt_mode==GUI_MODE_AMPLITUDE) {
    UB_Font_DrawString(GUI_AMP_XPOS,GUI_AMP_YPOS,buf,&Arial_14x22,RGB_COL_BLACK,GUI_COL_AKTIV);
  }
  else {
    UB_Font_DrawString(GUI_AMP_XPOS,GUI_AMP_YPOS,buf,&Arial_14x22,RGB_COL_BLACK,GUI_COL_NORMAL);
  }
}


//--------------------------------------------------------------
// interne Funktion
// anzeige vom Offset
//--------------------------------------------------------------
void p_menu_draw_OFF(void)
{
  static uint32_t old_len=99;

  sprintf(buf,"%d",DDS.offset);
  if(old_len!=strlen(buf)) {
    old_len=strlen(buf);
    UB_Font_DrawString(GUI_OFF_XPOS,GUI_OFF_YPOS,"    ",&Arial_14x22,RGB_COL_BLACK,GUI_COL_NORMAL);
  }

  if(GUI.akt_mode==GUI_MODE_OFFSET) {
    UB_Font_DrawString(GUI_OFF_XPOS,GUI_OFF_YPOS,buf,&Arial_14x22,RGB_COL_BLACK,GUI_COL_AKTIV);
  }
  else {
    UB_Font_DrawString(GUI_OFF_XPOS,GUI_OFF_YPOS,buf,&Arial_14x22,RGB_COL_BLACK,GUI_COL_NORMAL);
  }
}


