//--------------------------------------------------------------
// File     : gui.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_GUI_H
#define __STM32F4_UB_GUI_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32_ub_lcd_ili9341.h"
#include "stm32_ub_font.h"
#include "stm32_ub_touch_stmpe811.h"
#include "stm32_ub_graphic2d.h"
#include "stm32_ub_systick.h"
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "dds.h"


//--------------------------------------------------------------
// Color defines
//--------------------------------------------------------------
#define  GUI_COL_NORMAL    RGB_COL_WHITE
#define  GUI_COL_AKTIV     0x54DC



//--------------------------------------------------------------
// GUI defines (positionen der Texte)
//--------------------------------------------------------------
#define   GUI_HEADER_XP    220
#define   GUI_HEADER_YP      5
#define   GUI_VERSION_XP   220
#define   GUI_VERSION_YP   250
#define   GUI_FOOTER_XP      5
#define   GUI_FOOTER_YP      5


//--------------------------------------------------------------
#define   GUI_DELAY_MS     50  // refreshzeit der GUI/Touch


//--------------------------------------------------------------
// position der GUI
//--------------------------------------------------------------
#define  GUI_YPOS       10
#define  GUI_XPOS       215


//--------------------------------------------------------------
// position der Waveform-Texte
//--------------------------------------------------------------
#define  GUI_FORM_HEADER_XPOS       180
#define  GUI_FORM_HEADER_YPOS       5

#define  GUI_FORM_XPOS              155
#define  GUI_FORM_YPOS              5


//--------------------------------------------------------------
// position der FRQ-Texte
//--------------------------------------------------------------
#define  GUI_FRQ_HEADER_XPOS       120
#define  GUI_FRQ_HEADER_YPOS       5

#define  GUI_FRQ_XPOS              95
#define  GUI_FRQ_YPOS              5


//--------------------------------------------------------------
// position der Amplituden-Texte
//--------------------------------------------------------------
#define  GUI_AMP_HEADER_XPOS       60
#define  GUI_AMP_HEADER_YPOS       5

#define  GUI_AMP_XPOS              35
#define  GUI_AMP_YPOS              5


//--------------------------------------------------------------
// position der Offset-Texte
//--------------------------------------------------------------
#define  GUI_OFF_HEADER_XPOS       60
#define  GUI_OFF_HEADER_YPOS       110

#define  GUI_OFF_XPOS              35
#define  GUI_OFF_YPOS              110



//--------------------------------------------------------------
#define  GUI_REDRAW_BTN   0x01
#define  GUI_REDRAW_FRQ   0x02
#define  GUI_REDRAW_WAVE  0x04
#define  GUI_REDRAW_AMP   0x08
#define  GUI_REDRAW_OFF   0x10
#define  GUI_REDRAW_ALL   0xFF




//--------------------------------------------------------------
typedef enum {
  GUI_CHANGE_NONE =0,     // keine Aenderung
  GUI_CHANGE_FRQ,         // FRQ-Wert aendern
  GUI_CHANGE_AMP,         // Amplituden-Wert aendern
  GUI_CHANGE_OFF,         // Offset-Wert aendern 
  GUI_CHANGE_WAVE         // Kurvenform aendern
}GUI_CHANGE_t;


//--------------------------------------------------------------
typedef enum {
  GUI_MODE_WAVE =0,       // Kurvenform einstellen
  GUI_MODE_FRQ,           // FRQ einstellen
  GUI_MODE_AMPLITUDE,     // Amplitude einstellen
  GUI_MODE_OFFSET         // Offset einstellen
}GUI_MODE_t;



//--------------------------------------------------------------
// Button der GUI
//--------------------------------------------------------------
typedef enum {
  GUI_BTN_NONE =0,    // Button=keiner
  GUI_BTN_UP,         // Button=UP
  GUI_BTN_DOWN,       // Button=Down
  GUI_BTN_LEFT,       // Button=Left
  GUI_BTN_RIGHT       // Button=Right
}GUI_Button_t;


//--------------------------------------------------------------
// Struktur der "GUI"
//--------------------------------------------------------------
typedef struct {
  uint32_t redraw;
  GUI_MODE_t akt_mode;
  GUI_Button_t akt_button;
  GUI_Button_t old_button;
}GUI_t;
GUI_t GUI;



//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
void gui_init(void);
GUI_CHANGE_t gui_do(uint32_t redraw);



//--------------------------------------------------------------
#endif // __STM32F4_UB_GUI_H
